@@ -289,7 +289,8 @@ mu_str_expand (char **output, char const *input, mu_assoc_t assoc)
   ws.ws_closure = assoc;
 
   if (mu_wordsplit (input, &ws,
-		    MU_WRDSF_NOSPLIT | MU_WRDSF_GETVAR | MU_WRDSF_CLOSURE))
+		    MU_WRDSF_NOSPLIT | WRDSF_QUOTE |
+		    MU_WRDSF_GETVAR | MU_WRDSF_CLOSURE))
     {
       if (ws.ws_errno == MU_WRDSE_NOSPACE)
 	rc = ENOMEM;
