@@ -1,5 +1,5 @@
 /* wsp - test program for wordsplit
-   Copyright (C) 2014-2023 Sergey Poznyakoff
+   Copyright (C) 2014-2025 Sergey Poznyakoff
 
    Wordsplit is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -16,7 +16,7 @@
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
-#endif 
+#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -66,13 +66,13 @@ struct wsclosure
 			     the number of such variables. Forces the
 			     WRDSF_DOOFFS flag. */
   char *fenvbase[MAX_F_ENV+1];
-                          /* Environment for testing the ws_getenv function */
+			  /* Environment for testing the ws_getenv function */
   int fenvidx;            /* Number of variables in fenvbase */
 
   char *xenvbase[MAX_X_ENV+1];
-                          /* Extra environment variables */
+			  /* Extra environment variables */
   int xenvidx;            /* Number of variables in xenvbase */
-  
+
   int append_start;       /* First argument to append (index in argv) */
   int append_count;       /* Number of arguments to append */
 };
@@ -94,7 +94,7 @@ struct wsopt
   int tok;             /* Corresponding flag */
   int arg;             /* Option type (see the enum above) */
   void (*setfn) (int tok, int neg, char *arg, struct wsclosure *wsc);
-                       /* Setter function */
+		       /* Setter function */
 };
 
 /* Index of the next argument in the argv */
@@ -141,7 +141,7 @@ getwsopt (int argc, char **argv, struct wsopt *wso, struct wsclosure *wsc)
   if (strncmp (opt, "-D", 2) == 0)
     {
       char *asgn;
-      
+
       if (opt[2])
 	asgn = opt + 2;
       else if (wsoptind == argc)
@@ -161,8 +161,8 @@ getwsopt (int argc, char **argv, struct wsopt *wso, struct wsclosure *wsc)
 	}
       wsoptind--;
       return EOF;
-    }	  
-  
+    }
+
   if (strcmp (opt, "--version") == 0)
     {
       print_version ();
@@ -170,7 +170,7 @@ getwsopt (int argc, char **argv, struct wsopt *wso, struct wsclosure *wsc)
     }
 
   opt++; /* skip past initial dash */
-  
+
   if (strncmp (opt, "no-", 3) == 0)
     {
       negate = 1;
@@ -181,7 +181,7 @@ getwsopt (int argc, char **argv, struct wsopt *wso, struct wsclosure *wsc)
       negate = 1;
       opt += 2;
     }
-  
+
   for (; wso->name; wso++)
     {
       if (wso->arg == ws_boolean && wso->name[0] == 'n' && wso->name[1] == 'o'
@@ -221,8 +221,8 @@ getwsopt (int argc, char **argv, struct wsopt *wso, struct wsclosure *wsc)
 		}
 	      arg = argv[wsoptind++];
 	    }
-          else
-            arg = NULL;
+	  else
+	    arg = NULL;
 	  wso->setfn (wso->tok, negate, arg, wsc);
 	}
       return 0;
@@ -329,7 +329,7 @@ setfn_escape_word (int flag, int neg, char *arg, struct wsclosure *wsc)
 {
   setfn_escape_qw (arg, 0, wsc);
 }
-  
+
 static void
 setfn_escape_quote (int flag, int neg, char *arg, struct wsclosure *wsc)
 {
@@ -422,7 +422,7 @@ struct wsopt opttab[] = {
   { "trimnl",         TRIMNL_OPTION,        ws_boolean, setfn_global },
   { "plaintext",      PLAINTEXT_OPTION,     ws_boolean, setfn_global },
   { "env",            0,          ws_required_argument, setfn_env },
-  
+
   /* Wordsplit flags */
   { "append",         WRDSF_APPEND,         ws_boolean, setfn_flag },
   /*{ "reuse",  WRDSF_REUSE, ws_boolean, setfn_flag },*/
@@ -453,11 +453,12 @@ struct wsopt opttab[] = {
   { "bskeep_words",   WRDSO_BSKEEP_WORD,    ws_boolean, setfn_option },
   { "bskeep_quote",   WRDSO_BSKEEP_QUOTE,   ws_boolean, setfn_option },
   { "bskeep",         WRDSO_BSKEEP_WORD|WRDSO_BSKEEP_QUOTE,
-                                            ws_boolean, setfn_option },
+					    ws_boolean, setfn_option },
   { "novarsplit",     WRDSO_NOVARSPLIT,     ws_boolean, setfn_option },
   { "nocmdsplit",     WRDSO_NOCMDSPLIT,     ws_boolean, setfn_option },
   { "maxwords",       WRDSO_MAXWORDS, ws_required_argument, setfn_maxwords },
   { "namechar",       WRDSO_NAMECHAR, ws_required_argument, setfn_namechar },
+  { "retdelnotempty", WRDSO_RETDELNOTEMPTY, ws_boolean, setfn_option },
   /* String options */
   { "delim",          WRDSF_DELIM,  ws_required_argument, setfn_delim },
   { "comment",        WRDSF_COMMENT,ws_required_argument, setfn_comment },
@@ -465,11 +466,11 @@ struct wsopt opttab[] = {
   { "escape-word",    WRDSF_ESCAPE, ws_required_argument, setfn_escape_word },
   { "escape-quote",   WRDSF_ESCAPE, ws_required_argument, setfn_escape_quote },
 
-  { "dooffs",         WRDSF_DOOFFS, ws_multiple_arguments, setfn_dooffs },  
+  { "dooffs",         WRDSF_DOOFFS, ws_multiple_arguments, setfn_dooffs },
   { "append-args",    0,            ws_multiple_arguments, setfn_append },
 
   { "help",           0,                   ws_no_argument, setfn_help },
-  
+
   { NULL, 0 }
 };
 
@@ -477,7 +478,7 @@ static void
 help (void)
 {
   size_t i;
-  
+
   printf ("usage: %s [options] [-D VAR=VALUE ...] [VAR=VALUE...] [-- EXTRA...]\n", progname);
   printf ("options are:\n");
   for (i = 0; opttab[i].name; i++)
@@ -521,7 +522,7 @@ print_qword (const char *word, int plaintext)
 
   if (*word == 0)
     quote = 1;
-  
+
   if (size >= qlen)
     {
       qlen = size + 1;
@@ -542,7 +543,7 @@ make_env_kv (char **origenv)
 {
   size_t i, j, size;
   char **newenv;
-  
+
   /* Count the number of entries */
   for (i = 0; origenv[i]; i++)
     ;
@@ -604,13 +605,13 @@ cmd_quote (char **ret, const char *str, size_t len, char **argv)
   (*ret)[len] = 0;
   return WRDSE_OK;
 }
-  
+
 static int
 cmd_words (char **ret, const char *str, size_t len, char **argv)
 {
   char *p;
   int i;
-  
+
   p = malloc (len + 1);
   if (!p)
     return WRDSE_NOSPACE;
@@ -632,7 +633,7 @@ cmd_lines (char **ret, const char *str, size_t len, char **argv)
 {
   char *p;
   int i;
-  
+
   p = malloc (len + 1);
   if (!p)
     return WRDSE_NOSPACE;
@@ -664,7 +665,7 @@ wsp_runcmd (char **ret, const char *str, size_t len, char **argv, void *closure)
 {
   int i;
   char const msg[] = "unknown command: ";
-  
+
   for (i = 0; ; i++)
     {
       if (i == sizeof (comtab) / sizeof (comtab[0]))
@@ -680,11 +681,43 @@ wsp_runcmd (char **ret, const char *str, size_t len, char **argv, void *closure)
   return WRDSE_USERERR;
 }
 
+static void
+free_env (char **env)
+{
+  int i;
+  if (!env)
+    return;
+  for (i = 0; env[i]; i++)
+    free (env[i]);
+  free (env);
+}
+
+static void
+wsc_free_env (struct wsclosure *wsc)
+{
+  switch (wsc->env_type)
+    {
+    case env_null:
+    case env_none:
+      break;
+
+    case env_sys:
+      if (wsc->xenvidx || (wsc->wsflags & WRDSF_ENV_KV))
+	free_env ((char**)wsc->ws.ws_env);
+      break;
+
+    case env_extra:
+      if (wsc->wsflags & WRDSF_ENV_KV)
+	free_env ((char**)wsc->ws.ws_env);
+      break;
+    }
+}
+
 int
 main (int argc, char **argv)
 {
   struct wsclosure wsc;
-  char buf[1024], *ptr, *saved_ptr;
+  char buf[1024], *ptr, *saved_ptr = NULL;
   int next_call = 0;
 
   wsc.options = 0;
@@ -695,17 +728,17 @@ main (int argc, char **argv)
   wsc.xenvidx = 0;
   wsc.ws.ws_options = 0;
   wsc.wsflags = (WRDSF_DEFFLAGS & ~WRDSF_NOVAR) |
-                 WRDSF_ENOMEMABRT |
-                 WRDSF_SHOWERR;
+		 WRDSF_ENOMEMABRT |
+		 WRDSF_SHOWERR;
   wsc.append_count = 0;
-  
+
   progname = argv[0];
   while (getwsopt (argc, argv, opttab, &wsc) != EOF)
     ;
 
   wsc.fenvbase[wsc.fenvidx] = NULL;
   wsc.xenvbase[wsc.xenvidx] = NULL;
-  
+
   if (wsc.fenvidx > 0)
     {
       wsc.wsflags |= WRDSF_GETVAR | WRDSF_CLOSURE;
@@ -734,7 +767,7 @@ main (int argc, char **argv)
     case env_sys:
       {
 	char **newenv;
-	
+
 	if (wsc.xenvidx)
 	  {
 	    size_t i, j;
@@ -759,7 +792,11 @@ main (int argc, char **argv)
 
 	wsc.wsflags |= WRDSF_ENV;
 	if (wsc.wsflags & WRDSF_ENV_KV)
-	  wsc.ws.ws_env = (const char **) make_env_kv (newenv);
+	  {
+	    wsc.ws.ws_env = (const char **) make_env_kv (newenv);
+	    if (wsc.xenvidx)
+	      free_env (newenv);
+	  }
 	else
 	  wsc.ws.ws_env = (const char **) newenv;
       }
@@ -773,26 +810,26 @@ main (int argc, char **argv)
 	wsc.ws.ws_env = (const char **) wsc.xenvbase;
       break;
     }
-  
+
   if (!(wsc.wsflags & WRDSF_NOCMD))
     wsc.ws.ws_command = wsp_runcmd;
-  
+
   if (wsc.wsflags & WRDSF_INCREMENTAL)
     wsc.options |= TRIMNL_OPTION;
-  
+
   next_call = 0;
   while ((ptr = fgets (buf, sizeof (buf), stdin)))
     {
       int rc;
       size_t i;
-      
+
       if (wsc.options & TRIMNL_OPTION)
 	{
 	  size_t len = strlen (ptr);
 	  if (len && ptr[len-1] == '\n')
 	    ptr[len-1] = 0;
 	}
-      
+
       if (wsc.wsflags & WRDSF_INCREMENTAL)
 	{
 	  if (next_call)
@@ -810,15 +847,16 @@ main (int argc, char **argv)
 	      assert (ptr != NULL);
 	    }
 	}
-	
+
       rc = wordsplit (ptr, &wsc.ws, wsc.wsflags);
+      wsc.wsflags |= WRDSF_REUSE;
       if (rc)
 	{
 	  if (!(wsc.wsflags & WRDSF_SHOWERR))
 	    wordsplit_perror (&wsc.ws);
 	  continue;
 	}
-	  
+
       if (wsc.offarg)
 	{
 	  size_t i;
@@ -838,8 +876,7 @@ main (int argc, char **argv)
 	      continue;
 	    }
 	}
-      
-      wsc.wsflags |= WRDSF_REUSE;
+
       printf ("NF: %lu", (unsigned long) wsc.ws.ws_wordc);
       if (wsc.wsflags & WRDSF_DOOFFS)
 	printf (" (%lu)", (unsigned long) wsc.ws.ws_offs);
@@ -858,5 +895,9 @@ main (int argc, char **argv)
 	}
       printf ("TOTAL: %lu\n", (unsigned long) wsc.ws.ws_wordi);
     }
+  free (saved_ptr);
+  wsc_free_env (&wsc);
+  if (wsc.wsflags & WRDSF_REUSE)
+    wordsplit_free (&wsc.ws);
   return 0;
 }
