@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2024 Free Software Foundation, Inc.
+   Copyright (C) 1999-2025 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -53,7 +53,7 @@ static struct format_record format_table[] = {
   { "pop",  &mu_pop_record },
   { "pops",  &mu_pops_record },
   { "imap", &mu_imap_record },
-  { "imaps", &mu_imap_record },
+  { "imaps", &mu_imaps_record },
   { "sendmail", &mu_sendmail_record },
   { "smtp", &mu_smtp_record },
   { NULL, NULL },
