@@ -17,12 +17,12 @@
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
-#endif  
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>  
-#include <string.h>  
+#include <unistd.h>
+#include <string.h>
 #include <sieve-priv.h>
 
 int
@@ -47,6 +47,22 @@ mu_sieve_registry_require (mu_sieve_machine_t mach, const char *name,
   return 0;
 }
 
+int
+mu_sieve_registry_clone (mu_sieve_machine_t mach, mu_sieve_registry_t *reg)
+{
+  mu_sieve_registry_t *creg;
+  int rc;
+
+  if (reg->handle != NULL)
+    return mu_sieve_registry_require (mach, reg->name, reg->type);
+
+  rc = mu_i_sv_registry_add (mach, reg->name, &creg);
+  if (rc)
+    return rc;
+
+  *creg = *reg;
+  return 0;
+}
 
 static void
 regunload (void *data)
@@ -65,20 +81,18 @@ regcmp (void const *a, void const *b)
   return strcmp (rega->name, regb->name);
 }
 
-mu_sieve_registry_t *
-mu_sieve_registry_add (mu_sieve_machine_t mach, const char *name)
+int
+mu_i_sv_registry_add (mu_sieve_machine_t mach, const char *name,
+		      mu_sieve_registry_t **retreg)
 {
   mu_sieve_registry_t *reg;
   int rc;
-  
+
   if (!mach->registry)
     {
       rc = mu_list_create (&mach->registry);
       if (rc)
-	{
-	  mu_sieve_error (mach, "mu_list_create: %s", mu_strerror (rc));
-	  mu_sieve_abort (mach);
-	}
+	return rc;
       mu_list_set_destroy_item (mach->registry, regunload);
       mu_list_set_comparator (mach->registry, regcmp);
     }
@@ -88,9 +102,20 @@ mu_sieve_registry_add (mu_sieve_machine_t mach, const char *name)
   reg->required = 0;
   memset (&reg->v, 0, sizeof reg->v);
   rc = mu_list_append (mach->registry, reg);
+  if (rc == 0)
+    *retreg = reg;
+  return rc;
+}
+
+mu_sieve_registry_t *
+mu_sieve_registry_add (mu_sieve_machine_t mach, const char *name)
+{
+  mu_sieve_registry_t *reg;
+  int rc = mu_i_sv_registry_add (mach, name, &reg);
   if (rc)
     {
-      mu_sieve_error (mach, "mu_list_append: %s", mu_strerror (rc));
+      mu_sieve_error (mach, _("can't add to sieve registry: %s"),
+		      mu_strerror (rc));
       mu_sieve_abort (mach);
     }
   return reg;
@@ -102,7 +127,7 @@ mu_sieve_registry_lookup (mu_sieve_machine_t mach, const char *name,
 {
   mu_sieve_registry_t key, *reg;
   int rc;
-  
+
   key.name = name;
   key.type = type;
 
