@@ -70,7 +70,7 @@ struct _pop3_message
   int flags;
   mu_off_t offset;          /* Offset in the message cache stream */
   mu_off_t body_start;      /* Start of message, relative to offset */
-  mu_off_t body_end;        /* End of message, relative to offset */  
+  mu_off_t body_end;        /* End of message, relative to offset */
   size_t header_lines;      /* Number of lines in the header */
   size_t body_lines;        /* Number of lines in the body */
   int attr_flags;           /* Message attributes */
@@ -78,25 +78,25 @@ struct _pop3_message
   size_t message_lines;     /* Number of lines in the message */
   size_t num;               /* Message number */
   char *uidl;               /* Cached uidl string.  */
-  mu_message_t message;     /* Pointer to the message structure */ 
+  mu_message_t message;     /* Pointer to the message structure */
   struct _pop3_mailbox *mpd; /* Back pointer.  */
 };
-  
+
 struct _pop3_mailbox
 {
   mu_pop3_t pop3;             /* mu_pop3_t is the working horse */
   int pops;                   /* true if pop3 over SSL is being used */
   int is_updated;             /* true if the mailbox info is up to date */
-  
+
   size_t msg_count;           /* Number of messages in the mailbox */
   mu_off_t total_size;        /* Total mailbox size. */
   struct _pop3_message **msg; /* Array of messages */
-  size_t msg_max;             /* Actual size of the array */  
+  size_t msg_max;             /* Actual size of the array */
   mu_mailbox_t mbox;          /* MU mailbox corresponding to this one. */
 
   mu_stream_t cache;          /* Message cache stream */
    /* Temporary holders for user and passwd: */
-  char *user;    
+  char *user;
   mu_secret_t secret;
 };
 
@@ -119,9 +119,9 @@ pop_open (mu_mailbox_t mbox, int flags)
   /* Sanity checks. */
   if (mpd == NULL)
     return EINVAL;
-  
+
   mbox->flags = flags;
-  
+
   memset (&hints, 0, sizeof (hints));
   hints.flags = MU_AH_DETECT_FAMILY;
   hints.port = mpd->pops ? MU_POPS_PORT : MU_POP_PORT;
@@ -130,7 +130,7 @@ pop_open (mu_mailbox_t mbox, int flags)
   status = mu_sockaddr_from_url (&sa, mbox->url, &hints);
   if (status)
     return status;
-      
+
   status = mu_tcp_stream_create_from_sa (&stream, sa, NULL, mbox->flags);
   if (status)
     {
@@ -168,14 +168,14 @@ pop_open (mu_mailbox_t mbox, int flags)
     }
   mu_pop3_set_carrier (mpd->pop3, stream);
   mu_stream_unref (stream);
-  
+
   if (mu_debug_level_p (MU_DEBCAT_MAILBOX, MU_DEBUG_PROT))
     mu_pop3_trace (mpd->pop3, MU_POP3_TRACE_SET);
   if (mu_debug_level_p (MU_DEBCAT_MAILBOX, MU_DEBUG_TRACE6))
     mu_pop3_trace_mask (mpd->pop3, MU_POP3_TRACE_SET, MU_XSCRIPT_SECURE);
   if (mu_debug_level_p (MU_DEBCAT_MAILBOX, MU_DEBUG_TRACE7))
     mu_pop3_trace_mask (mpd->pop3, MU_POP3_TRACE_SET, MU_XSCRIPT_PAYLOAD);
-    
+
   do
     {
       status = mu_pop3_connect (mpd->pop3);
@@ -183,17 +183,17 @@ pop_open (mu_mailbox_t mbox, int flags)
 	break;
 
       status = mu_pop3_capa (mpd->pop3, 1, NULL);
-      if (status == MU_ERR_REPLY) 
-        {
-	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR, 
+      if (status == MU_ERR_REPLY)
+	{
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
 		    ("server rejected the CAPA command: %s",
 		     mu_pop3_strresp (mpd->pop3)));
 	  /* try to continue anyway */
-        } 
+	}
       else if (status)
 	return status;
 
-#ifdef WITH_TLS      
+#ifdef WITH_TLS
       if (!mpd->pops &&
 	  mu_url_sget_param (mbox->url, "notls", NULL) == MU_ERR_NOENT &&
 	  mu_pop3_capa_test (mpd->pop3, "STLS", NULL) == 0)
@@ -206,18 +206,18 @@ pop_open (mu_mailbox_t mbox, int flags)
       status = mu_authority_authenticate (mbox->folder->authority);
     }
   while (0);
-  
+
   if (status)
     mu_pop3_destroy (&mpd->pop3);
   return status;
 }
-  
+
 static int
 pop_close (mu_mailbox_t mbox)
 {
   struct _pop3_mailbox *mpd = mbox->data;
   int status;
-  
+
   status = mu_pop3_quit (mpd->pop3);
   if (status)
     mu_error ("mu_pop3_quit failed: %s", mu_strerror (status));
@@ -227,7 +227,7 @@ pop_close (mu_mailbox_t mbox)
   if (mpd->msg)
     {
       size_t i;
-      
+
       mu_monitor_wrlock (mbox->monitor);
       /* Destroy the pop messages and resources associated to them.  */
       for (i = 0; i < mpd->msg_count; i++)
@@ -269,14 +269,14 @@ pop_is_updated (mu_mailbox_t mbox)
     return 0;
   return mpd->is_updated;
 }
-      
+
 /* Return the number of messages in the mailbox */
 static int
 pop_messages_count (mu_mailbox_t mbox, size_t *pcount)
 {
   struct _pop3_mailbox *mpd = mbox->data;
   int status;
-  
+
   if (mpd == NULL)
     return EINVAL;
 
@@ -306,7 +306,7 @@ pop_scan (mu_mailbox_t mbox, size_t msgno, size_t *pcount)
   struct _pop3_mailbox *mpd = mbox->data;
   int flags;
   mu_iterator_t itr;
-  
+
   status = pop_messages_count (mbox, &count);
   if (status != 0)
     return status;
@@ -320,7 +320,7 @@ pop_scan (mu_mailbox_t mbox, size_t msgno, size_t *pcount)
   status = mu_pop3_list_all (mpd->pop3, &itr);
   if (status)
     return status;
-  
+
   for (i = 0, mu_iterator_first (itr);
        i <= count && !mu_iterator_is_done (itr);
        i++, mu_iterator_next (itr))
@@ -328,7 +328,7 @@ pop_scan (mu_mailbox_t mbox, size_t msgno, size_t *pcount)
       const char *str;
       char *p;
       size_t num;
-      
+
       mu_iterator_current (itr, (void**) &str);
       num = strtoul (str, &p, 10);
 
@@ -374,15 +374,15 @@ pop_scan (mu_mailbox_t mbox, size_t msgno, size_t *pcount)
 	    }
 	}
     }
-  
+
   mu_iterator_destroy (&itr);
 
   if (mbox->observable)
     {
       /* MU_EVT_MESSAGE_ADD must be delivered only when it is already possible
-         to retrieve the message in question.  It could not be done in the
-         main loop because no other pop3d function can be called while LIST
-         is being handled.  Hence the extra loop. */
+	 to retrieve the message in question.  It could not be done in the
+	 main loop because no other pop3d function can be called while LIST
+	 is being handled.  Hence the extra loop. */
       for (i = 0; i <= count; i++)
 	{
 	  if (mu_observable_notify (mbox->observable, MU_EVT_MESSAGE_ADD,
@@ -390,7 +390,7 @@ pop_scan (mu_mailbox_t mbox, size_t msgno, size_t *pcount)
 	    break;
 	}
     }
-  
+
   return status;
 }
 
@@ -423,6 +423,61 @@ pop_get_size (mu_mailbox_t mbox, mu_off_t *psize)
   return status;
 }
 
+static int
+pop_get_uidls (mu_mailbox_t mbox, mu_list_t list)
+{
+  struct _pop3_mailbox *mpd = mbox->data;
+  mu_iterator_t itr;
+  int status = mu_pop3_uidl_all (mpd->pop3, &itr);
+  if (status)
+    return status;
+
+  for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
+       mu_iterator_next (itr))
+    {
+      const char *str;
+      char *p;
+      size_t num;
+      struct mu_uidl *uidl;
+
+      mu_iterator_current (itr, (void**) &str);
+
+      num = strtoul (str, &p, 10);
+      if (*p != ' ')
+	{
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		    ("invalid reply to UIDL command: %s", str));
+	  status = MU_ERR_BADREPLY;
+	  break;
+	}
+      p++;
+
+      uidl = malloc (sizeof (*uidl));
+      if (!uidl)
+	{
+	  status = ENOMEM;
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		    ("out of memory when processing the UIDL response"));
+	  break;
+	}
+
+      uidl->msgno = num;
+      strncpy (uidl->uidl, p, sizeof (uidl->uidl));
+
+      if ((status = mu_list_append (list, uidl)) != 0)
+	{
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		    ("error appending to list when processing the UIDL"
+		     " response: %s", mu_strerror (status)));
+	  free (uidl);
+	  break;
+	}
+    }
+  mu_iterator_destroy (&itr);
+  if (status)
+    mu_list_clear (list);
+  return status;
+}
 
 /* ------------------------------------------------------------------------- */
 /* POP3 message streams */
@@ -436,18 +491,18 @@ pop_stream_drain (mu_stream_t str)
   while (mu_stream_read (str, buf, sizeof buf, &size) == 0 && size)
     ;
 }
-     
+
 static int
 _pop_message_get_stream (struct _pop3_message *mpm, mu_stream_t *pstr)
 {
   int status;
   struct _pop3_mailbox *mpd = mpm->mpd;
-  
+
   if (!(mpm->flags & _POP3_MSG_CACHED))
     {
       mu_stream_t stream;
       mu_off_t size;
-      
+
       status = mu_pop3_retr (mpd->pop3, mpm->num, &stream);
       if (status)
 	return status;
@@ -455,7 +510,7 @@ _pop_message_get_stream (struct _pop3_message *mpm, mu_stream_t *pstr)
       do
 	{
 	  mu_stream_t flt;
-	  
+
 	  if (!mpd->cache)
 	    {
 	      status = mu_temp_stream_create (&mpd->cache, 0);
@@ -505,7 +560,7 @@ pop_message_get_stream (mu_message_t msg, mu_stream_t *pstr)
   struct _pop3_message *mpm = mu_message_get_owner (msg);
   return _pop_message_get_stream (mpm, pstr);
 }
-        
+
 static int
 pop_scan_message (struct _pop3_message *mpm)
 {
@@ -515,11 +570,11 @@ pop_scan_message (struct _pop3_message *mpm)
 
   if (mpm->flags & _POP3_MSG_SCANNED)
     return 0;
-  
+
   status = _pop_message_get_stream (mpm, &stream);
   if (status)
     return status;
-      
+
   scan.flags = MU_SCAN_SEEK | MU_SCAN_SIZE;
   scan.message_start = 0;
   scan.message_size = mpm->message_size;
@@ -540,7 +595,7 @@ pop_scan_message (struct _pop3_message *mpm)
 
       mpm->flags |= _POP3_MSG_SCANNED;
     }
-  
+
   return status;
 }
 
@@ -603,7 +658,7 @@ pop_create_message (struct _pop3_message *mpm, struct _pop3_mailbox *mpd)
 {
   int status;
   mu_message_t msg;
-  
+
   status = mu_message_create (&msg, mpm);
   if (status)
     return status;
@@ -620,7 +675,7 @@ pop_create_pop3_message (struct _pop3_mailbox *mpd, size_t msgno,
 			 struct _pop3_message **mptr)
 {
   struct _pop3_message *mpm;
-  
+
   if (msgno > mpd->msg_count)
     return MU_ERR_NOENT;
 
@@ -643,7 +698,7 @@ pop_create_pop3_message (struct _pop3_mailbox *mpd, size_t msgno,
   /* Back pointer.  */
   mpm->mpd = mpd;
   mpm->num = msgno;
-  
+
   mpd->msg[msgno - 1] = mpm;
   *mptr = mpm;
   return 0;
@@ -664,11 +719,11 @@ pop_header_blurb (mu_stream_t stream, size_t maxlines,
   char *buf = NULL;
   size_t n;
   size_t nlines = 0;
-  
+
   status = mu_opool_create (&opool, MU_OPOOL_DEFAULT);
   if (status)
     return status;
-      
+
   while ((status = mu_stream_getline (stream, &buf, &size, &n)) == 0 && n > 0)
     {
       size_t len = mu_rtrim_class (buf, MU_CTYPE_ENDLN);
@@ -679,7 +734,7 @@ pop_header_blurb (mu_stream_t stream, size_t maxlines,
       if (maxlines && ++nlines >= maxlines)
 	break;
     }
-      
+
   if (status == 0)
     {
       n = mu_opool_size (opool);
@@ -824,7 +879,7 @@ pop_create_attribute (struct _pop3_message *mpm)
 {
   int status;
   mu_attribute_t attribute;
-  
+
   status = mu_attribute_create (&attribute, mpm);
   if (status)
     return status;
@@ -906,7 +961,7 @@ pop_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pnwriten)
   struct _pop3_message *mpm = mu_message_get_owner (msg);
   struct _pop3_mailbox *mpd = mpm->mpd;
   size_t len;
-  
+
   if (!mpm->uidl)
     {
       if (mu_pop3_capa_test (mpd->pop3, "UIDL", NULL) == 0)
@@ -970,7 +1025,7 @@ pop_get_message (mu_mailbox_t mbox, size_t msgno, mu_message_t *pmsg)
   status = pop_create_message (mpm, mpd);
   if (status)
     return status;
-  
+
   do
     {
       status = pop_create_header (mpm);
@@ -982,7 +1037,7 @@ pop_get_message (mu_mailbox_t mbox, size_t msgno, mu_message_t *pmsg)
       status = pop_create_body (mpm);
     }
   while (0);
-  
+
   if (status)
     {
       mu_message_destroy (&mpm->message, mpm);
@@ -1007,13 +1062,13 @@ pop_expunge (mu_mailbox_t mbox)
   int status = 0;
   size_t i;
   size_t expcount = 0;
-  
+
   if (mpd == NULL)
     return EINVAL;
 
   if (!mpd->msg)
     return 0;
-  
+
   for (i = 0; i < mpd->msg_count; i++)
     {
       struct _pop3_message *mpm = mpd->msg[i];
@@ -1050,7 +1105,7 @@ _pop3_mailbox_init (mu_mailbox_t mbox, int pops)
     return ENOMEM;
 
   mpd->pop3 = NULL;
-  
+
   mpd->mbox = mbox;		/* Back pointer.  */
   mpd->pops = pops;
 
@@ -1067,7 +1122,8 @@ _pop3_mailbox_init (mu_mailbox_t mbox, int pops)
   mbox->_scan = pop_scan;
   mbox->_message_unseen = pop_message_unseen;
   mbox->_get_size = pop_get_size;
-  
+  mbox->_get_uidls = pop_get_uidls;
+
   /* Messages.  */
   mbox->_get_message = pop_get_message;
   mbox->_expunge = pop_expunge;
@@ -1123,7 +1179,7 @@ pop_get_user (mu_authority_t auth)
     status = mu_ticket_get_cred (ticket, mbox->url, "Pop User: ",
 				 &mpd->user, NULL);
   if (status == MU_ERR_NOENT || mpd->user == NULL)
-    return MU_ERR_NOUSERNAME;  
+    return MU_ERR_NOUSERNAME;
   return status;
 }
 
@@ -1157,7 +1213,7 @@ _pop_apop (mu_authority_t auth)
   mu_mailbox_t mbox = folder->data;
   struct _pop3_mailbox *mpd = mbox->data;
   int status;
-  
+
   status = pop_get_user (auth);
   if (status)
     return status;
@@ -1183,7 +1239,7 @@ _pop_user (mu_authority_t auth)
   mu_mailbox_t mbox = folder->data;
   struct _pop3_mailbox *mpd = mbox->data;
   int status;
-  
+
   status = pop_get_user (auth);
   if (status)
     return status;
@@ -1192,7 +1248,7 @@ _pop_user (mu_authority_t auth)
     {
       /* Fetch the secret from them.  */
       status = pop_get_passwd (auth);
-      if (status == 0) 
+      if (status == 0)
 	{
 	  status = mu_pop3_pass (mpd->pop3, mu_secret_password (mpd->secret));
 	  mu_secret_password_unref (mpd->secret);
