@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2024 Free Software Foundation, Inc.
+   Copyright (C) 1999-2025 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -2331,7 +2331,7 @@ mh_fvm_destroy (mh_fvm_t *fvmp)
       mu_stream_unref (fvm->output);
       
       free (fvm);
-      *fvmp = fvm;
+      *fvmp = NULL;
     }
 }
 
